@@ -1,7 +1,7 @@
 /*
     libDriveIo - MMC drive interrogation library
 
-    Copyright (C) 2007-2019 GuinpinSoft inc <libdriveio@makemkv.com>
+    Copyright (C) 2007-2025 GuinpinSoft inc <libdriveio@makemkv.com>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -22,6 +22,7 @@
 #include <stdint.h>
 #include <driveio/scsihlp.h>
 #include <driveio/error.h>
+#include <lgpl/byteorder.h>
 #include <string.h>
 #include <unistd.h>
 
@@ -103,7 +104,7 @@ int LibDriveIo::ExecuteReadScsiCommand(ISimpleScsiTarget* ScsiTarget,const uint8
 
     if (res.Status!=0)
     {
-        return DRIVEIO_ERR_SCSI_STATUS(res.Status);
+        return ScsiErrorFromResult(&res);
     }
 
     *BufferSize = res.Transferred;
@@ -125,7 +126,7 @@ int LibDriveIo::ExecuteWriteScsiCommand(ISimpleScsiTarget* ScsiTarget,const uint
 
     if (res.Status!=0)
     {
-        return DRIVEIO_ERR_SCSI_STATUS(res.Status);
+        return ScsiErrorFromResult(&res);
     }
 
     if (res.Transferred!=BufferSize)
@@ -232,6 +233,8 @@ static void AddInquiryData(ScsiInquiryData *InquiryData,const uint8_t* Data)
 
 static void AddFirmwareDate(ScsiDriveInfo *DriveInfo,const uint8_t* Data)
 {
+    unsigned int len = Data[3];
+    if (len<14) return;
     CopyPaddedString(DriveInfo->FirmwareDate,14,Data+4);
 }
 
@@ -255,9 +258,15 @@ static int ReadSingleFeatureDescriptor(ISimpleScsiTarget* ScsiTarget,uint16_t Id
 
     len = BufferLen;
     err=LibDriveIo::ExecuteReadScsiCommand(ScsiTarget,cdb_rf,10,Buffer,&len);
+    if (DRIVEIO_ERROR_TYPE(err) == DRIVEIO_ERROR_TYPE(DRIVEIO_ERR_SCSI_SENSE(0)))
+    {
+        // Some drives do error out on mandatory commands...
+        *Have = false;
+        return 0;
+    }
     if (err) return err;
 
-    slen = uint32_get_be(Buffer) + 4;
+    slen = rd32be(Buffer) + 4;
     if (slen>len) slen = len;
 
     if ( (slen > BufferLen) || (slen < (8+2)) )
@@ -266,7 +275,7 @@ static int ReadSingleFeatureDescriptor(ISimpleScsiTarget* ScsiTarget,uint16_t Id
         return 0;
     }
 
-    code = uint16_get_be(Buffer+8);
+    code = rd16be(Buffer+8);
     if (code != Id)
     {
         *Have = false;
@@ -277,7 +286,7 @@ static int ReadSingleFeatureDescriptor(ISimpleScsiTarget* ScsiTarget,uint16_t Id
     return 0;
 }
 
-int LibDriveIo::QueryInquiryInfo(ISimpleScsiTarget* ScsiTarget,uint8_t Evpd,uint8_t *Buffer,unsigned int *BufferSize)
+int LibDriveIo::QueryInquiryInfo(ISimpleScsiTarget* ScsiTarget,uint8_t *Buffer,unsigned int *BufferSize)
 {
     int err;
     unsigned int len,slen;
@@ -286,43 +295,27 @@ int LibDriveIo::QueryInquiryInfo(ISimpleScsiTarget* ScsiTarget,uint8_t Evpd,uint
 
     memset(cdb_inquiry,0,sizeof(cdb_inquiry));
     cdb_inquiry[0]=0x12;
-    if (Evpd!=0)
-    {
-        cdb_inquiry[1]=1;
-        cdb_inquiry[2]=Evpd;
-    }
 
-    err=ExecuteReadScsiCommand(ScsiTarget,cdb_inquiry,6,Buffer,254,&res);
+    err=ExecuteReadScsiCommand(ScsiTarget,cdb_inquiry,6,Buffer,0x60,&res);
     if (err) return err;
     len = res.Transferred;
 
-    if (Evpd==0)
+    if (res.Status != 0) return DRIVEIO_ERR_SCSI_STATUS(res.Status);
+    slen = ((unsigned int)Buffer[4]) + 5;
+    if (slen > len)
     {
-        if (res.Status!=0) return DRIVEIO_ERR_SCSI_STATUS(res.Status);
-        slen = ((unsigned int)Buffer[4])+5;
-        if (slen>len)
+        if (len >= 35)
         {
-            if (len>=35)
-            {
-                //
-                // Some drives return incomplete inquiry data, accept it
-                // as long as mandatory portion is present
-                //
-                memset(Buffer+len,0,slen-len);
-            } else {
-                return DRIVEIO_ERROR_BAD_DATA;
-            }
+            //
+            // Some drives return incomplete inquiry data, accept it
+            // as long as mandatory portion is present
+            //
+            memset(Buffer + len, 0, slen - len);
+        } else {
+            return DRIVEIO_ERROR_BAD_DATA;
         }
-    } else {
-        *BufferSize=0;
-        if (res.Status!=0)
-        {
-            return 0;
-        }
-        slen = ((unsigned int)Buffer[3])+4;
-        if (slen>len) return 0;
-        if (Buffer[1]!=Evpd) return 0;
     }
+
     *BufferSize = slen;
     return 0;
 }
@@ -340,7 +333,7 @@ int LibDriveIo::BuildInquiryData(ISimpleScsiTarget* ScsiTarget,DIO_INFOLIST List
         AddInquiryData(InquiryData,(const uint8_t*)item.Data);
     } else {
         // no drive info, query all data manually
-        if (0!=(err=QueryInquiryInfo(ScsiTarget,0,buf,&len))) return err;
+        if (0!=(err=QueryInquiryInfo(ScsiTarget,buf,&len))) return err;
         AddInquiryData(InquiryData,buf);
 
         if ((buf[0]&0x1f)!=5) return DRIVEIO_ERROR_BAD_DATA; // not a MMC drive
@@ -465,6 +458,23 @@ int LibDriveIo::ScsiErrorFromResult(const ScsiCmdResponse *CmdResult)
         }
     }
 
+    if ((CmdResult->Status == 0xff) &&
+        (CmdResult->SenseLen == 7) &&
+        (CmdResult->SenseData[0]==0xfe))
+    {
+        uint16_t wrap_type = rd16be(CmdResult->SenseData + 1);
+        uint32_t wrap_code = rd32be(CmdResult->SenseData + 1 + 2);
+
+        switch (wrap_type)
+        {
+        case 1: return (0xc0000000 | wrap_code);
+        default:
+        case 2:
+            return DRIVEIO_IO_ERROR(wrap_code);
+        }
+    }
+
+
     return DRIVEIO_ERR_SCSI_STATUS(CmdResult->Status);
 }
 
