@@ -48,10 +48,10 @@ START_LIBEBML_NAMESPACE
 /*!
   \todo handle more than CodedSize of 5
 */
-int CodedSizeLength(uint64 Length, unsigned int SizeLength, bool bSizeFinite)
+int CodedSizeLength(uint64 Length, unsigned int SizeLength, bool bSizeIsFinite)
 {
   unsigned int CodedSize;
-  if (bSizeFinite) {
+  if (bSizeIsFinite) {
     // prepare the head of the size (000...01xxxxxx)
     // optimal size
     if (Length < 127) // 2^7 - 1
@@ -214,18 +214,20 @@ EbmlCallbacks::EbmlCallbacks(EbmlElement & (*Creator)(), const EbmlId & aGlobalI
   ,DebugName(aDebugName)
   ,Context(aContext)
 {
-  assert((Create!=NULL) || !strcmp(aDebugName, "DummyElement"));
+  assert((Create!=nullptr) || !strcmp(aDebugName, "DummyElement"));
 }
 
 const EbmlSemantic & EbmlSemanticContext::GetSemantic(size_t i) const
 {
   assert(i<Size);
-  if (i<Size)
+  /* if (i<Size) */
     return MyTable[i];
 
+  /*
   buf::string ss;
   ss.format(128, "EbmlSemanticContext::GetSemantic: programming error: index i outside of table size (%u >= %u)", (unsigned int)i, (unsigned int)Size);
   throw mkv_logic_error(ss);
+  */
 }
 
 
@@ -242,19 +244,6 @@ EbmlElement::EbmlElement(uint64 aDefaultSize, bool bValueSet)
   Size = DefaultSize;
 }
 
-EbmlElement::EbmlElement(const EbmlElement & ElementToClone)
-  :Size(ElementToClone.Size)
-  ,DefaultSize(ElementToClone.DefaultSize)
-  ,SizeLength(ElementToClone.SizeLength)
-  ,bSizeIsFinite(ElementToClone.bSizeIsFinite)
-  ,ElementPosition(ElementToClone.ElementPosition)
-  ,SizePosition(ElementToClone.SizePosition)
-  ,bValueIsSet(ElementToClone.bValueIsSet)
-  ,DefaultIsSet(ElementToClone.DefaultIsSet)
-  ,bLocked(ElementToClone.bLocked)
-{
-}
-
 EbmlElement::~EbmlElement()
 {
   assert(!bLocked);
@@ -282,14 +271,13 @@ EbmlElement * EbmlElement::FindNextID(IOCallback & DataStream, const EbmlCallbac
     aElementPosition = DataStream.getFilePointer();
     uint32 ReadSize = 0;
     BitMask = 1 << 7;
-    while (1) {
-      ReadSize += DataStream.read(&PossibleId[PossibleID_Length], 1);
-      if (ReadSize == uint32(PossibleID_Length)) {
-        return NULL; // no more data ?
-      }
-      if (++PossibleID_Length > 4) {
-        return NULL; // we don't support element IDs over class D
-      }
+    while (PossibleID_Length < 4) {
+      if (!DataStream.read(&PossibleId[PossibleID_Length], 1))
+        return nullptr;            // no more data
+
+      ++ReadSize;
+      ++PossibleID_Length;
+
       if (PossibleId[0] & BitMask) {
         // this is the last octet of the ID
         // check wether that's the one we're looking for
@@ -304,13 +292,16 @@ EbmlElement * EbmlElement::FindNextID(IOCallback & DataStream, const EbmlCallbac
       BitMask >>= 1;
     }
 
+    if (!bElementFound)
+      return nullptr;
+
     // read the data size
     aSizePosition = DataStream.getFilePointer();
     uint32 _SizeLength;
     do {
       if (PossibleSizeLength >= 8)
         // Size is larger than 8 bytes
-        return NULL;
+        return nullptr;
 
       ReadSize += DataStream.read(&PossibleSize[PossibleSizeLength++], 1);
       _SizeLength = PossibleSizeLength;
@@ -318,7 +309,7 @@ EbmlElement * EbmlElement::FindNextID(IOCallback & DataStream, const EbmlCallbac
     } while (_SizeLength == 0);
   }
 
-  EbmlElement *Result = NULL;
+  EbmlElement *Result = nullptr;
   EbmlId PossibleID(PossibleId, PossibleID_Length);
   if (PossibleID == EBML_INFO_ID(ClassInfos)) {
     // the element is the one expected
@@ -326,8 +317,8 @@ EbmlElement * EbmlElement::FindNextID(IOCallback & DataStream, const EbmlCallbac
   } else {
     /// \todo find the element in the context
     Result = new (std::nothrow) EbmlDummy(PossibleID);
-    if(Result == NULL)
-      return NULL;
+    if(Result == nullptr)
+      return nullptr;
   }
 
   Result->SetSizeLength(PossibleSizeLength);
@@ -336,7 +327,7 @@ EbmlElement * EbmlElement::FindNextID(IOCallback & DataStream, const EbmlCallbac
 
   if (!Result->ValidateSize() || (SizeFound != SizeUnknown && MaxDataSize < Result->Size)) {
     delete Result;
-    return NULL;
+    return nullptr;
   }
 
   // check if the size is not all 1s
@@ -346,7 +337,7 @@ EbmlElement * EbmlElement::FindNextID(IOCallback & DataStream, const EbmlCallbac
     if (!Result->SetSizeInfinite()) {
       /// \todo the element is not allowed to be infinite
       delete Result;
-      return NULL;
+      return nullptr;
     }
   } else Result->SetSizeInfinite(false);
   Result->ElementPosition = aElementPosition;
@@ -370,11 +361,12 @@ EbmlElement * EbmlElement::FindNextElement(IOCallback & DataStream, const EbmlSe
   int PossibleSizeLength;
   uint64 SizeUnknown;
   int ReadIndex = 0; // trick for the algo, start index at 0
-  uint32 ReadSize = 0;
+  uint32 ReadSize = 0, IdStart = 0;
   uint64 SizeFound;
   int SizeIdx;
   bool bFound;
   int UpperLevel_original = UpperLevel;
+  uint64 ParseStart = DataStream.getFilePointer();
 
   do {
     // read a potential ID
@@ -400,18 +392,21 @@ EbmlElement * EbmlElement::FindNextElement(IOCallback & DataStream, const EbmlSe
         // ID not found
         // shift left the read octets
         memmove(&PossibleIdNSize[0],&PossibleIdNSize[1], --ReadIndex);
+        IdStart++;
       }
 
+      if (MaxDataSize <= ReadSize)
+        break;
       if (DataStream.read(&PossibleIdNSize[ReadIndex++], 1) == 0) {
-        return NULL; // no more data ?
+        return nullptr; // no more data ?
       }
       ReadSize++;
 
-    } while (!bFound && MaxDataSize > ReadSize);
+    } while (!bFound);
 
     if (!bFound)
       // we reached the maximum we could read without a proper ID
-      return NULL;
+      return nullptr;
 
     SizeIdx = ReadIndex;
     ReadIndex -= PossibleID_Length;
@@ -419,7 +414,7 @@ EbmlElement * EbmlElement::FindNextElement(IOCallback & DataStream, const EbmlSe
     // read the data size
     uint32 _SizeLength;
     PossibleSizeLength = ReadIndex;
-    while (1) {
+    while (true) {
       _SizeLength = PossibleSizeLength;
       SizeFound = ReadCodedSizeValue(&PossibleIdNSize[PossibleID_Length], _SizeLength, SizeUnknown);
       if (_SizeLength != 0) {
@@ -430,8 +425,12 @@ EbmlElement * EbmlElement::FindNextElement(IOCallback & DataStream, const EbmlSe
         bFound = false;
         break;
       }
+      if (MaxDataSize <= ReadSize) {
+        bFound = false;
+        break;
+      }
       if( DataStream.read( &PossibleIdNSize[SizeIdx++], 1 ) == 0 ) {
-        return NULL; // no more data ?
+        return nullptr; // no more data ?
       }
       ReadSize++;
       PossibleSizeLength++;
@@ -442,7 +441,7 @@ EbmlElement * EbmlElement::FindNextElement(IOCallback & DataStream, const EbmlSe
       EbmlId PossibleID(PossibleIdNSize, PossibleID_Length);
       EbmlElement * Result = CreateElementUsingContext(PossibleID, Context, UpperLevel, false, AllowDummyElt, MaxLowerLevel);
       ///< \todo continue is misplaced
-      if (Result != NULL) {
+      if (Result != nullptr) {
         if (AllowDummyElt || !Result->IsDummy()) {
           Result->SetSizeLength(_SizeLength);
 
@@ -452,16 +451,15 @@ EbmlElement * EbmlElement::FindNextElement(IOCallback & DataStream, const EbmlSe
           //  0 : child
           //  1 : same level
           //  + : further parent
-          if (Result->ValidateSize() && (SizeFound == SizeUnknown || UpperLevel > 0 || MaxDataSize == 0 || MaxDataSize >= (PossibleID_Length + PossibleSizeLength + SizeFound))) {
-            if (SizeFound == SizeUnknown) {
-              Result->SetSizeInfinite();
+          if (Result->ValidateSize() && (SizeFound == SizeUnknown || UpperLevel > 0 || MaxDataSize == 0 ||
+                                         MaxDataSize >= (IdStart + PossibleID_Length + _SizeLength + SizeFound))) {
+            if (SizeFound != SizeUnknown || Result->SetSizeInfinite()) {
+              Result->ElementPosition = ParseStart + IdStart;
+              Result->SizePosition = Result->ElementPosition + PossibleID_Length;
+              // place the file at the beggining of the data
+              DataStream.setFilePointer(Result->SizePosition + _SizeLength);
+              return Result;
             }
-
-            Result->SizePosition = DataStream.getFilePointer() - SizeIdx + EBML_ID_LENGTH(PossibleID);
-            Result->ElementPosition = Result->SizePosition - EBML_ID_LENGTH(PossibleID);
-            // place the file at the beggining of the data
-            DataStream.setFilePointer(Result->SizePosition + _SizeLength);
-            return Result;
           }
         }
         delete Result;
@@ -471,10 +469,11 @@ EbmlElement * EbmlElement::FindNextElement(IOCallback & DataStream, const EbmlSe
     // recover all the data in the buffer minus one byte
     ReadIndex = SizeIdx - 1;
     memmove(&PossibleIdNSize[0], &PossibleIdNSize[1], ReadIndex);
+    IdStart++;
     UpperLevel = UpperLevel_original;
-  } while ( MaxDataSize > DataStream.getFilePointer() - SizeIdx + PossibleID_Length );
+  } while ( MaxDataSize >= ReadSize );
 
-  return NULL;
+  return nullptr;
 }
 
 /*!
@@ -482,9 +481,9 @@ EbmlElement * EbmlElement::FindNextElement(IOCallback & DataStream, const EbmlSe
 */
 EbmlElement * EbmlElement::SkipData(EbmlStream & DataStream, const EbmlSemanticContext & Context, EbmlElement * TestReadElt, bool AllowDummyElt)
 {
-  EbmlElement * Result = NULL;
+  EbmlElement * Result = nullptr;
   if (bSizeIsFinite) {
-    assert(TestReadElt == NULL);
+    assert(TestReadElt == nullptr);
     assert(ElementPosition < SizePosition);
     DataStream.I_O().setFilePointer(SizePosition + CodedSizeLength(Size, SizeLength, bSizeIsFinite) + Size, seek_beginning);
     //    DataStream.I_O().setFilePointer(Size, seek_current);
@@ -493,34 +492,34 @@ EbmlElement * EbmlElement::SkipData(EbmlStream & DataStream, const EbmlSemanticC
     // read elements until an upper element is found
     /////////////////////////////////////////////////
     bool bEndFound = false;
-    while (!bEndFound && Result == NULL) {
+    while (!bEndFound && Result == nullptr) {
       // read an element
       /// \todo 0xFF... and true should be configurable
       //      EbmlElement * NewElt;
-      if (TestReadElt == NULL) {
+      if (TestReadElt == nullptr) {
         int bUpperElement = 0; // trick to call FindNextID correctly
         Result = DataStream.FindNextElement(Context, bUpperElement, 0xFFFFFFFFL, AllowDummyElt);
       } else {
         Result = TestReadElt;
-        TestReadElt = NULL;
+        TestReadElt = nullptr;
       }
 
-      if (Result != NULL) {
+      if (Result != nullptr) {
         unsigned int EltIndex;
         // data known in this Master's context
         for (EltIndex = 0; EltIndex < EBML_CTX_SIZE(Context); EltIndex++) {
           if (EbmlId(*Result) == EBML_CTX_IDX_ID(Context,EltIndex)) {
             // skip the data with its own context
-            Result = Result->SkipData(DataStream, EBML_SEM_CONTEXT(EBML_CTX_IDX(Context,EltIndex)), NULL);
+            Result = Result->SkipData(DataStream, EBML_SEM_CONTEXT(EBML_CTX_IDX(Context,EltIndex)), nullptr);
             break; // let's go to the next ID
           }
         }
 
         if (EltIndex >= EBML_CTX_SIZE(Context)) {
-          if (EBML_CTX_PARENT(Context) != NULL) {
+          if (EBML_CTX_PARENT(Context) != nullptr) {
             Result = SkipData(DataStream, *EBML_CTX_PARENT(Context), Result);
           } else {
-            assert(Context.GetGlobalContext != NULL);
+            assert(Context.GetGlobalContext != nullptr);
             if (Context != Context.GetGlobalContext()) {
               Result = SkipData(DataStream, Context.GetGlobalContext(), Result);
             } else {
@@ -541,7 +540,7 @@ EbmlElement *EbmlElement::CreateElementUsingContext(const EbmlId & aID, const Eb
                                                     int & LowLevel, bool IsGlobalContext, bool bAllowDummy, unsigned int MaxLowerLevel)
 {
   unsigned int ContextIndex;
-  EbmlElement *Result = NULL;
+  EbmlElement *Result = nullptr;
 
   // elements at the current level
   for (ContextIndex = 0; ContextIndex < EBML_CTX_SIZE(Context); ContextIndex++) {
@@ -551,30 +550,30 @@ EbmlElement *EbmlElement::CreateElementUsingContext(const EbmlId & aID, const Eb
   }
 
   // global elements
-  assert(Context.GetGlobalContext != NULL); // global should always exist, at least the EBML ones
+  assert(Context.GetGlobalContext != nullptr); // global should always exist, at least the EBML ones
   const EbmlSemanticContext & tstContext = Context.GetGlobalContext();
   if (tstContext != Context) {
     LowLevel--;
     MaxLowerLevel--;
     // recursive is good, but be carefull...
     Result = CreateElementUsingContext(aID, tstContext, LowLevel, true, bAllowDummy, MaxLowerLevel);
-    if (Result != NULL) {
+    if (Result != nullptr) {
       return Result;
     }
     LowLevel++;
     MaxLowerLevel++;
   } else {
-    return NULL;
+    return nullptr;
   }
 
   // parent elements
-  if (EBML_CTX_MASTER(Context) != NULL && aID == EBML_INFO_ID(*EBML_CTX_MASTER(Context))) {
+  if (EBML_CTX_MASTER(Context) != nullptr && aID == EBML_INFO_ID(*EBML_CTX_MASTER(Context))) {
     LowLevel++; // already one level up (same as context)
     return &EBML_INFO_CREATE(*EBML_CTX_MASTER(Context));
   }
 
   // check wether it's not part of an upper context
-  if (EBML_CTX_PARENT(Context) != NULL) {
+  if (EBML_CTX_PARENT(Context) != nullptr) {
     LowLevel++;
     MaxLowerLevel++;
     return CreateElementUsingContext(aID, *EBML_CTX_PARENT(Context), LowLevel, IsGlobalContext, bAllowDummy, MaxLowerLevel);
@@ -663,8 +662,8 @@ bool EbmlElement::CompareElements(const EbmlElement *A, const EbmlElement *B)
 {
   if (EbmlId(*A) == EbmlId(*B))
     return A->IsSmallerThan(B);
-  else
-    return false;
+
+  return false;
 }
 
 #ifndef EBML_NO_READ
@@ -709,20 +708,19 @@ filepos_t EbmlElement::OverwriteHead(IOCallback & output, bool bKeepPosition)
 
 filepos_t EbmlElement::OverwriteData(IOCallback & output, bool bKeepPosition)
 {
-	if (ElementPosition == 0) {
-		return 0; // the element has not been written
-	}
+  if (ElementPosition == 0) {
+    return 0; // the element has not been written
+  }
 
-	uint64 HeaderSize = EbmlId(*this).GetLength() + CodedSizeLength(Size, SizeLength, bSizeIsFinite);
+  auto HeaderSize = EbmlId(*this).GetLength() + CodedSizeLength(Size, SizeLength, bSizeIsFinite);
+  auto DataSize   = GetSize();
 
-	filepos_t DataSize = GetSize();
-
-	uint64 CurrentPosition = output.getFilePointer();
-	output.setFilePointer(GetElementPosition()+HeaderSize);
-	filepos_t Result = RenderData(output,true,bKeepPosition);
-	output.setFilePointer(CurrentPosition);
-	MKV_ASSERT(Result==DataSize);
-	return Result;
+  auto CurrentPosition = output.getFilePointer();
+  output.setFilePointer(GetElementPosition() + HeaderSize);
+  auto Result = RenderData(output, true, bKeepPosition);
+  output.setFilePointer(CurrentPosition);
+  assert(Result == DataSize);
+  return Result;
 }
 
 
