@@ -19,11 +19,13 @@ static const unsigned int AP_UIMSG_BOXERROR=516;
 static const unsigned int AP_UIMSG_BOXWARNING=1028;
 static const unsigned int AP_UIMSG_BOXYESNO=776;
 static const unsigned int AP_UIMSG_BOXYESNO_ERR=1288;
+static const unsigned int AP_UIMSG_BOXYESNO_REG=1544;
 static const unsigned int AP_UIMSG_YES=0;
 static const unsigned int AP_UIMSG_NO=1;
 static const unsigned int AP_UIMSG_DEBUG=32;
 static const unsigned int AP_UIMSG_HIDDEN=64;
 static const unsigned int AP_UIMSG_EVENT=128;
+static const unsigned int AP_UIMSG_HAVE_URL=131072;
 static const unsigned int AP_UIMSG_VITEM_BASE=5200;
 static const unsigned int AP_MMBD_DISC_FLAG_BUSENC=2;
 static const unsigned int AP_MMBD_MMBD_DISC_FLAG_AACS=4;
@@ -50,6 +52,10 @@ static const unsigned int AP_vastr_WebSiteURL=18;
 static const unsigned int AP_vastr_OpenDVDFileFilter=19;
 static const unsigned int AP_vastr_DefaultSelectionString=20;
 static const unsigned int AP_vastr_DefaultOutputFileName=21;
+static const unsigned int AP_vastr_ExternalAppItem=22;
+static const unsigned int AP_vastr_InterfaceLanguage=23;
+static const unsigned int AP_vastr_ProfileString=24;
+static const unsigned int AP_vastr_KeyString=25;
 //
 typedef enum _AP_ItemAttributeId
 {
@@ -124,6 +130,10 @@ static const unsigned int AP_DriveStateLoading=3;
 
 static const unsigned int AP_Notify_UpdateLayoutFlag_NoTime=1;
 static const unsigned int AP_ProgressCurrentIndex_SourceName=65280;
+static const unsigned int AP_BackupFlagDecryptVideo=1;
+static const unsigned int AP_OpenFlagManualMode=1;
+static const unsigned int AP_UpdateDrivesFlagNoScan=1;
+static const unsigned int AP_UpdateDrivesFlagNoSingleDrive=2;
 
 
 static const unsigned int AP_AVStreamFlag_DirectorsComments=1;
@@ -138,8 +148,7 @@ static const unsigned int AP_AVStreamFlag_ProfileSecondaryStream=16384;
 static const unsigned int AP_AVStreamFlag_OffsetSequenceIdPresent=32768;
 
 
-static const unsigned int AP_APP_LOC_POGZ=1;
-static const unsigned int AP_APP_LOC_MAX=100;
+static const unsigned int AP_APP_LOC_MAX=7000;
 
 
 static const unsigned long APP_DUMP_DONE_PARTIAL=5004;
@@ -149,11 +158,13 @@ static const unsigned long APP_ASK_FOLDER_CREATE=5013;
 static const unsigned long APP_FOLDER_INVALID=5016;
 static const unsigned long PROGRESS_APP_SAVE_MKV_FREE_SPACE=5033;
 static const unsigned long PROT_DEMO_KEY_EXPIRED=5021;
+static const unsigned long APP_KEYTYPE_INVALID=5095;
 static const unsigned long APP_EVAL_TIME_NEVER=5067;
 static const unsigned long APP_BACKUP_FAILED=5069;
 static const unsigned long APP_BACKUP_COMPLETED=5070;
 static const unsigned long APP_BACKUP_COMPLETED_HASHFAIL=5079;
 static const unsigned long PROFILE_NAME_DEFAULT=5086;
+static const unsigned long VITEM_NAME=5202;
 static const unsigned long VITEM_TIMESTAMP=5223;
 static const unsigned long APP_IFACE_TITLE=6000;
 static const unsigned long APP_CAPTION_MSG=6001;
@@ -259,9 +270,6 @@ static const unsigned long APP_IFACE_SETTINGS_IO_SINGLE_DRIVE=6168;
 static const unsigned long APP_IFACE_SETTINGS_DVD_AUTO=6085;
 static const unsigned long APP_IFACE_SETTINGS_DVD_MIN_LENGTH=6086;
 static const unsigned long APP_IFACE_SETTINGS_DVD_SP_REMOVE=6087;
-static const unsigned long APP_IFACE_SETTINGS_STREAM_ENABLE_UPNP=6161;
-static const unsigned long APP_IFACE_SETTINGS_STREAM_BIND_IP=6162;
-static const unsigned long APP_IFACE_SETTINGS_STREAM_BIND_PORT=6163;
 static const unsigned long APP_IFACE_SETTINGS_AACS_KEY_DIR=6088;
 static const unsigned long APP_IFACE_SETTINGS_BDP_MISC=6129;
 static const unsigned long APP_IFACE_SETTINGS_BDP_DUMP_ALWAYS=6130;
@@ -272,10 +280,12 @@ static const unsigned long APP_IFACE_SETTINGS_DEST_TYPE_CUSTOM=6092;
 static const unsigned long APP_IFACE_SETTINGS_DESTDIR=6093;
 static const unsigned long APP_IFACE_SETTINGS_GENERAL_MISC=6094;
 static const unsigned long APP_IFACE_SETTINGS_LOG_DEBUG_MSG=6095;
-static const unsigned long APP_IFACE_SETTINGS_ENABLE_SITE_ACCESS=6149;
 static const unsigned long APP_IFACE_SETTINGS_DATA_DIR=6167;
 static const unsigned long APP_IFACE_SETTINGS_EXPERT_MODE=6169;
 static const unsigned long APP_IFACE_SETTINGS_SHOW_AVSYNC=6170;
+static const unsigned long APP_IFACE_SETTINGS_GENERAL_ONLINE_UPDATES=6188;
+static const unsigned long APP_IFACE_SETTINGS_ENABLE_INTERNET_ACCESS=6187;
+static const unsigned long APP_IFACE_SETTINGS_PROXY_SERVER=6189;
 static const unsigned long APP_IFACE_SETTINGS_TAB_GENERAL=6096;
 static const unsigned long APP_IFACE_SETTINGS_MSG_FAILED=6097;
 static const unsigned long APP_IFACE_SETTINGS_MSG_RESTART=6098;
@@ -293,6 +303,13 @@ static const unsigned long APP_IFACE_SETTINGS_ADV_DEFAULT_SELECTION=6174;
 static const unsigned long APP_IFACE_SETTINGS_ADV_EXTERN_EXEC_PATH=6175;
 static const unsigned long APP_IFACE_SETTINGS_PROT_JAVA_PATH=6177;
 static const unsigned long APP_IFACE_SETTINGS_ADV_OUTPUT_FILE_NAME_TEMPLATE=6178;
+static const unsigned long APP_IFACE_SETTINGS_TAB_INTEGRATION=6190;
+static const unsigned long APP_IFACE_SETTINGS_INT_TEXT=6191;
+static const unsigned long APP_IFACE_SETTINGS_INT_HDR_PATH=6192;
+static const unsigned long APP_IFACE_KEY_TEXT=6179;
+static const unsigned long APP_IFACE_KEY_NAME=6182;
+static const unsigned long APP_IFACE_KEY_TYPE=6183;
+static const unsigned long APP_IFACE_KEY_DATE=6184;
 static const unsigned long APP_IFACE_BACKUPDLG_TEXT_CAPTION=6137;
 static const unsigned long APP_IFACE_BACKUPDLG_TEXT=6138;
 static const unsigned long APP_IFACE_BACKUPDLG_FOLDER=6139;
@@ -411,6 +428,7 @@ typedef enum _ApSettingId
   apset_path_DestDirMRU ,
   apset_app_DefaultOutputFileName ,
   apset_sdf_Stop ,
+  apset_app_Proxy ,
   apset_MaxValue
 } ApSettingId;
 
