@@ -1,7 +1,7 @@
 /*
     MakeMKV GUI - Graphics user interface application for MakeMKV
 
-    Copyright (C) 2007-2019 GuinpinSoft inc <makemkvgui@makemkv.com>
+    Copyright (C) 2007-2025 GuinpinSoft inc <makemkvgui@makemkv.com>
 
     You may use this file in accordance with the end user license
     agreement provided with the Software. For licensing terms and
@@ -21,18 +21,15 @@
 #include <sys/mount.h>
 #endif
 
-uint64_t get_free_space(const utf16_t* Folder)
+uint64_t get_free_space(const utf8_t* Folder)
 {
     uint64_t fspace;
     struct statfs st;
-    QString folder_str;
-
-    folder_str = QStringFromUtf16(Folder);
 
     fspace=0;
     fspace--;
 
-    if (statfs((const char*)folder_str.toUtf8(),&st))
+    if (statfs(Folder,&st))
     {
         return 0;
     }
